import sys
import os
import cocos
import pyglet

from creator import SceneGraph as scenegraph 
from creator import AnyNode as anynode  
from creator import Sprite as CC_Sprite 
from creator import SpriteType as spritetype
from creator import SpriteSizeMode as sizemode
from creator import Label as CC_Label
from creator import LabelOverflowType as CC_LabelOverFlowType
from creator import Node as CC_Node
from creator import Button as CC_Button

"""
Play the animation with name anim_name if its' available on node_name
TODO: figure out if we can optimize this to reuse one action with deep copy to play on new node.
"""
def PlayNodeAnimation(scn, node_name, anim_name, index=0):
    if not(node_name in scn.node_dict):
        print "Could not find the node: ", node_name
        return        
    node = scn.node_dict[node_name][index]
    if anim_name in node.anim_dict:
        anim = node.anim_dict[anim_name]
        node.do(anim)
    else:
        print "Could not find animation ", anim_name

def SetButtonCallback(scn, node_name, callback, index=0):
    if not(node_name in scn.node_dict):
        print "Could not find the node: ", node_name
        return       
    node = scn.node_dict[node_name][index]
    if isinstance(node, cocos.menu.ImageMenuItem):
        node.callback_func = callback

def dummy_quit():
    print "Done with world"
    sys.exit(0)
    return True

def get_seq_action(key_frames, anim_type):
    ANIM_TYPE_SCALE_X = 0
    ANIM_TYPE_SCALE_Y = 1
    ANIM_TYPE_COLOR = 2
    ANIM_TYPE_POSITION = 3
    ANIM_TYPE_ROTATION = 4
    ANIM_TYPE_OPACITY = 5

    if len(key_frames) > 0:
        print anim_type, key_frames
        prev_frame = 0
        actions = None
        for i in xrange(len(key_frames)):
            key_frame = key_frames[i]
            action = None
            if anim_type == ANIM_TYPE_SCALE_X or anim_type == ANIM_TYPE_SCALE_Y:
                action = cocos.actions.interval_actions.ScaleTo(key_frame[1], key_frame[0] - prev_frame)
            elif anim_type == ANIM_TYPE_OPACITY:
                action = cocos.actions.interval_actions.FadeTo(key_frame[1], key_frame[0] - prev_frame)
            elif anim_type == ANIM_TYPE_ROTATION:
                action = cocos.actions.interval_actions.RotateTo(key_frame[1], key_frame[0] - prev_frame)
            elif anim_type == ANIM_TYPE_POSITION:
                pos = (key_frame[1].X(), key_frame[1].Y())
                action = cocos.actions.interval_actions.MoveTo(pos, key_frame[0] - prev_frame)
            
            #if the animation is something we don't support yet we should move on. 
            if action == None:
                continue
            if actions == None:
                actions = action
            else:
                actions += action
            prev_frame = key_frame[0]
        return actions
    return None

def parseAnimProps(props):
    #create a list of function pointers for us to get values. 
    len_functions = [props.ScaleXLength, props.ScaleYLength, props.ColorLength, props.PositionLength, props.RotationLength, props.OpacityLength]
    get_functions = [props.ScaleX, props.ScaleY, props.Color, props.Position, props.Rotation, props.Opacity]
    prop_names = ["scalex", "scaley", "color", "position", "rotation", "opacity"]
 
    spawn_actions = None
    for i in xrange(len(prop_names)):
        len_func = len_functions[i]
        get_func = get_functions[i]
        props_len = len_func()
        key_frames = []

        #gather keyfames for the animation on prop
        for j in xrange(props_len):
            key_frame  = get_func(j) 
            key_frames.append((key_frame.Frame(), key_frame.Value()))

        #create a seqence of actions for the keyframes and return
        seq_action = get_seq_action(key_frames, i)
        
        if seq_action and spawn_actions == None:
            spawn_actions = seq_action
        elif seq_action:
            spawn_actions |= seq_action
    return spawn_actions

#get this function to return an action that will do the full animation.
#set the name to match the name of the animation clip
def parseAnimation(animRef):
    anim_dict = dict()
    
    if animRef == None:
        return anim_dict

    print 'NumAnimations:' + str(animRef.ClipsLength())
    for i in xrange(animRef.ClipsLength()):
        clip = animRef.Clips(i)
        curve_len = clip.CurveDataLength()
        print clip.Name(), "CurveLen:" + str(curve_len)
        actions = None
        #TODO: figureout what the curve_len does 
        for j in xrange(curve_len):
            anim_props = clip.CurveData(j).Props() 
            actions = parseAnimProps(anim_props)
            if actions:
                anim_dict[clip.Name()] = actions
    return anim_dict
        
        

def assignNodeInfo(node, cc_node):
    pos = cc_node.Position()
    node.position = pos.X(), pos.Y()
    anchor = cc_node.AnchorPoint()
    anim_dict = parseAnimation(cc_node.Anim())
    node.anim_dict = anim_dict
    # print anim_dict, cc_node.AnchorPoint().X(), cc_node.AnchorPoint().Y()
    # node.anchor_x = cc_node.AnchorPoint().X() * cc_node.ContentSize().W()    
    # node.anchor_y = cc_node.AnchorPoint().Y() * cc_node.ContentSize().H()
    node.scale_x = cc_node.ScaleX()
    node.scale_y = cc_node.ScaleY()
    #node.scale = scale[0]
    if isinstance(node, cocos.sprite.Sprite):
        col = cc_node.Color()
        node.color = (col.R(), col.G(), col.B())
        node.opacity = cc_node.Opacity()
        print "SpriteOpacity: ", node.opacity
    elif isinstance(node, cocos.text.Label):
        col = cc_node.Color()
        node.color = (col.R(), col.G(), col.B(), cc_node.Opacity())
        node.opacity = cc_node.Opacity()
        print "LabelColor: ", node.color

def createButton(buff, pos):
    btn = CC_Button.Button()
    btn.Init(buff, pos)
    print btn.SpriteFrameName(), btn.PressedSpriteFrameName(), btn.DisabledSpriteFrameName()
    im = cocos.menu.ImageMenuItem(btn.SpriteFrameName(), None)
    assignNodeInfo(im, btn.Node())
    return (im, btn.Node().Name())
    
def createSprite(buff, pos):
    #if type is 1 or if the framename is "default_sprite_splash.png" we can use the color layer. s
    spr = CC_Sprite.Sprite()
    spr.Init(buff, pos)
    print spr.SpriteFrameName() + " Type: " +  str(spr.SpriteType())

    node = None
    col = spr.Node().Color()
    if spr.SpriteType() == spritetype.SpriteType.Simple and not(spr.SizeMode() == sizemode.SpriteSizeMode.Custom):
        node = cocos.sprite.Sprite(os.path.join("Resources", spr.SpriteFrameName()))
        assignNodeInfo(node, spr.Node())
    else:#use colored layer for anything other than normal sprite.
        #TODO: fix for 9 patch image. which might be a common usecase.
        size = spr.Node().ContentSize()
        node = cocos.layer.util_layers.ColorLayer(col.R(), col.G(), col.B(), spr.Node().Opacity(), int(size.W()), int(size.H()))        
        assignNodeInfo(node, spr.Node())
        pos = node.position
        node.position = (pos[0]-size.W()/2, pos[1]-size.H()/2)
    return (node, spr.Node().Name())

def createLabel(buff, pos):
    lbl = CC_Label.Label() 
    lbl.Init(buff, pos)
    node = lbl.Node()
    pos = node.Position()
    print("\t" + lbl.LabelText() + "(" + str(pos.X()) + "," + str(pos.Y()) + ")", lbl.FontSize(), "OverFlowType:", lbl.OverflowType())

    ml = False
    if lbl.OverflowType() == CC_LabelOverFlowType.LabelOverflowType.ResizeHeight:
        ml = True

    label = cocos.text.Label(lbl.LabelText(),
        font_name=lbl.FontName(),
        font_size=lbl.FontSize(),
        anchor_x='center', anchor_y='center', multiline=ml, width=lbl.Node().ContentSize().W(), height = lbl.Node().ContentSize().H())
    assignNodeInfo(label, lbl.Node())
    return (label, node.Name())

def createNode(buff, pos):
    node = CC_Node.Node()
    node.Init(buff, pos)
    n = cocos.layer.Layer()
    assignNodeInfo(n, node)
    return (n, node.Name())

class CreatorReader:    
    BUTTONS_LIST_KEY = "_cc_buttons"
    def __init__(self):
        #name of the animation -> (object_ref, action_ref)
        self.file_path = ""
        self.node_dict = dict()
        self.design_resolution = (0, 0)

    def createTree(self, tree):
        nodeType = tree.ObjectType()
        buff = tree.Object().Bytes
        pos = tree.Object().Pos
        node = None
        node_name = ""

        if nodeType == anynode.AnyNode.Scene:
            print("Scene")
            node = cocos.scene.Scene()
            node_name = "scene"
        elif nodeType == anynode.AnyNode.Label:
            print("Label")
            (node, node_name) = createLabel(buff, pos)
        elif nodeType == anynode.AnyNode.Sprite:
            print("Sprite")
            (node, node_name) = createSprite(buff, pos)
        elif nodeType == anynode.AnyNode.Node:
            print("Node")
            (node, node_name) = createNode(buff, pos)
        elif nodeType == anynode.AnyNode.Button:
            print("Button")
            (node, node_name) = createButton(buff, pos)            
        else:
            print ("UnSupported Node")
        
        if node == None:
            print ("Skipping")
            return None

        #if its a button, add it to the list of buttons. 
        if nodeType == anynode.AnyNode.Button:
            if CreatorReader.BUTTONS_LIST_KEY in self.node_dict:
                self.node_dict[CreatorReader.BUTTONS_LIST_KEY].append(node)
            else:
                self.node_dict[CreatorReader.BUTTONS_LIST_KEY] = [node]

        #cocos creator allows objects with same name, we should to.
        if node_name in self.node_dict:
            self.node_dict[node_name].append(node)
        else:
            self.node_dict[node_name] = [node,]

        num_children = tree.ChildrenLength()
        for i in xrange(num_children):
            sub_tree = self.createTree(tree.Children(i))
            if not sub_tree == None and not(isinstance(sub_tree, cocos.menu.ImageMenuItem)):
                node.add(sub_tree)

        if nodeType == anynode.AnyNode.Scene and CreatorReader.BUTTONS_LIST_KEY in self.node_dict:
            #create a menu for holding the menuItems. 
            menu = cocos.menu.Menu()            
            menu.position  = self.design_resolution[0]/2, self.design_resolution[1]/2
            menu.on_quit = dummy_quit

            position_list = []
            button_list = self.node_dict[CreatorReader.BUTTONS_LIST_KEY]
            for btn in button_list:      
                pos = btn.position
                btn.position = (0, 0)
                position_list.append(pos)

            layout = cocos.menu.fixedPositionMenuLayout(position_list)
            menu.create_menu(items = button_list, layout_strategy = layout, unselected_effect=None, selected_effect=None, activated_effect = cocos.menu.shake())

            #hack to make buttons behave as expected (We don't want the whole selected / not selected sharades.)
            for btn in button_list:
                btn.item.scale = btn.selected_item.scale
                btn.item.color = btn.selected_item.color

            node.add(menu)

        return node

    def LoadScene(self, path_to_file):
        #returns a scene node
        buf = pyglet.resource.file(path_to_file, "rb").read()
        #buf = open(sys.argv[1], 'rb').read()
        buf = bytearray(buf)
        sg = scenegraph.SceneGraph.GetRootAsSceneGraph(buf, 0)
        size = sg.DesignResolution()
        print size.W(), size.H()
        self.design_resolution = (size.W(), size.H())
        root = sg.Root()
        if root.ObjectType() == anynode.AnyNode.Scene:
            scene = self.createTree(root)
            scene.node_dict = self.node_dict
            return scene
        else:
            print "Please provide a scene file to load"

if __name__ == "__main__":
    cocos.director.director.init(960, 640)
    pyglet.resource.path.append("./Resources")
    pyglet.resource.reindex()
    print pyglet.resource.path
    cr = CreatorReader()
    #load a scene.
    scn = cr.LoadScene(sys.argv[1])
    #set callback for button.     
    SetButtonCallback(scn, "New Button", dummy_quit)
    PlayNodeAnimation(scn, "label", "RotTest")

    cocos.director.director.run(scn)