import cocos
import pyglet
import math
from cocos.actions.interval_actions import *
from cocos.actions.instant_actions import *
import questions

class OpenCard:
    OPEN_CARD = "OpenCard"
    CLOSED_CARD = "ClosedCard"
    OPTION_TEXT = "option_text"
    HEADER = "oc_header"
    QUESTION_TEXT = "QuestionText"
    NAME_TEXT = "CharacterName"
    CHARACTER = "character"
    CIRCLE = "circle"
    CARD_LEFT = -1
    CARD_MIDDLE = 0
    CARD_RIGHT = 1
    
    def done_cardflip(self):
        self.disable_interation = False

    def newcard_anim(self):
        self.init_card()
        #set the opacity of the newly created elements. 
        self.question_text.opacity = 0
        self.name_text.opacity = 0

        self.card.opacity = 255
        self.character.opacity = 255
        self.card.scale = 0
        self.card.rotation = 0
        self.card.do(ScaleTo(1, 0.25) + Delay(0.5) + CallFunc(self.done_cardflip))
        self.question_text.do(Delay(0.25) + FadeIn(0.25))
        self.name_text.do(Delay(0.25) + FadeIn(0.25))
        self.question_text.element.align = "center"

    def done_falloff(self):
        print ("Playing Flip Animation")
        self.question = self.gp.get_question()
        self.newcard_anim()

    def falloff_anim(self):
        self.set_circle_scale()
        dest_angle = self.card.rotation * 4
        duration = 0.25
        self.disable_interation = True
        self.card.do(RotateTo(dest_angle, duration) | FadeOut(duration))
        self.option_text.do(FadeOut(duration))
        self.header.do(FadeOut(duration) + Delay(0.25) + CallFunc(self.done_falloff))
        self.character.do(FadeOut(duration))
        self.question_text.do(FadeOut(duration))
        self.name_text.do(FadeOut(duration))

    def __init__(self, gp):
        self.card = gp.get_node(OpenCard.OPEN_CARD)
        self.header = gp.get_node(OpenCard.HEADER)
        self.option_text = gp.get_node(OpenCard.OPTION_TEXT)
        self.question_text = gp.get_node(self.QUESTION_TEXT)
        self.name_text = gp.get_node(self.NAME_TEXT)
        self.state = self.CARD_MIDDLE
        self.card.transform_anchor_x = 0
        self.card.transform_anchor_y = -self.card.height/2
        self.header.opacity = 0
        self.option_text.opacity = 0
        self.disable_interation = False
        self.gp = gp
        self.question = gp.get_question()
        self.circles = gp.get_nodes(OpenCard.CIRCLE)
        self.character = gp.get_node(self.CHARACTER)
        self.init_card()

    def set_circle_scale(self, scales = [0, 0, 0]):
        _scales = map(math.fabs, scales)
        for i, circle in enumerate(self.circles):
            circle.scale_x = _scales[i] * 0.125
            circle.scale_y = _scales[i] * 0.125

    def init_card(self):
        self.update_label_text(self.name_text, self.question[questions.CHARACTER_KEY].upper())
        self.update_label_text(self.question_text, self.question[questions.QUESTION_KEY])
        image_path = "Texture/" + self.question[questions.CHARACTER_KEY] + ".png"
        self.character.image = pyglet.resource.image(image_path)

        #TODO: update the image on card

    def update_label_text(self, label, text):
        label.element.text = text
        #label.kwargs['text'] = text
        #label.create_element()

    def on_mouse_press(self, x, y, buttons, modifiers):
        if self.disable_interation: 
            return

        if buttons & pyglet.window.mouse.LEFT and not(self.card.are_actions_running()):
            if self.state == self.CARD_LEFT:
                print ("Left side chosen")
                self.gp.player_choice(False)
                self.falloff_anim()
            elif self.state == self.CARD_RIGHT:
                print ("Right side Choosen")
                self.gp.player_choice(True)
                self.falloff_anim()
            else:
                print ("No Choice yet")

    def on_mouse_motion(self, x, y, dx, dy):
        if self.disable_interation:
            return

        #TODO: get rid of the hard coded value.
        x_mid = 960/2.0
        y_mid = 640/2.0
        angle = (x-x_mid)/10.0

        #TODO: get rid of the magic numbers
        if math.fabs(angle) < 10:
            self.card.rotation = angle
            self.option_text.opacity = 0
            self.header.opacity = 0
            if not(self.state == self.CARD_MIDDLE):
                self.state = self.CARD_MIDDLE
                self.set_circle_scale()
                #TODO: fade out the header and label                
        elif angle < -10 and not(self.state == self.CARD_LEFT):
            left_option = self.question[questions.OPTIONS_KEY][0]
            values = left_option[questions.VALUES_KEY]
            self.set_circle_scale(values)            
            self.update_label_text(self.option_text, left_option[questions.OPTIONS_KEY])            
            self.option_text.opacity = 0
            self.card.do(RotateTo(-15, 0.5))
            self.header.do(FadeIn(0.75))
            self.option_text.do(FadeIn(1))
            self.state = self.CARD_LEFT
        elif angle > 10 and not(self.state == self.CARD_RIGHT):
            right_option = None
            try:
                right_option = self.question[questions.OPTIONS_KEY][1]
            except: 
                right_option = self.question[questions.OPTIONS_KEY][0]
            self.set_circle_scale(right_option[questions.VALUES_KEY])
            self.update_label_text(self.option_text, right_option[questions.OPTIONS_KEY])
            self.option_text.opacity = 0
            self.card.do(RotateTo(15, 0.5))
            self.state = self.CARD_RIGHT
            self.header.do(FadeIn(0.75))
            self.option_text.do(FadeIn(1))
